--[[
Copyright (C) GtX (Andy), 2021

Author: GtX | Andy
Date: 19.01.2021
Revision: FS22-01

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No modifications may be made to this script, including conversion to other game versions without written permission from GtX | Andy
Copying or removing any part of this code for external use without written permission from GtX | Andy is prohibited.

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
Ohne schriftliche Genehmigung von GtX | Andy dürfen keine Änderungen an diesem Skript vorgenommen werden, einschließlich der Konvertierung in andere Spielversionen
Das Kopieren oder Entfernen irgendeines Teils dieses Codes zur externen Verwendung ohne schriftliche Genehmigung von GtX | Andy ist verboten.
]]


SleeperCab = {}

SleeperCab.MOD_NAME = g_currentModName
SleeperCab.SPEC_NAME = string.format("spec_%s.sleeperCab", g_currentModName)

function SleeperCab.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Enterable, specializations) and
        not SpecializationUtil.hasSpecialization(Locomotive, specializations) and
        not SpecializationUtil.hasSpecialization(Rideable, specializations)
end

function SleeperCab.initSpecialization()
    local schema = Vehicle.xmlSchema

    schema:setXMLSpecializationType("SleeperCab")

    schema:register(XMLValueType.STRING, "vehicle.enterable.sleeperCab.requiredConfiguration#name", "Configuration name required indexes")
    schema:register(XMLValueType.INT, "vehicle.enterable.sleeperCab.requiredConfiguration.configuration(?)#index", "Configuration index to allow sleeping")
    schema:register(XMLValueType.VECTOR_TRANS, "vehicle.enterable.sleeperCab.requiredConfiguration.configuration(?)#cameraTranslation", "Camera translation when index active (Optional)")
    schema:register(XMLValueType.VECTOR_ROT, "vehicle.enterable.sleeperCab.requiredConfiguration.configuration(?)#cameraRotation", "Camera rotation when index active (Optional)")

    schema:register(XMLValueType.NODE_INDEX, "vehicle.enterable.sleeperCab#cameraNode", "Camera node (Optional, if not provided then the normal sleep animation will be used)", nil, false)
    schema:register(XMLValueType.BOOL, "vehicle.enterable.sleeperCab#deactivateEngineOnSleep", "Should deactivate vehicle engine when sleeping starts", true)
    schema:register(XMLValueType.BOOL, "vehicle.enterable.sleeperCab#deactivateLightsOnSleep", "Should deactivate vehicle lights when sleeping starts", true)

    schema:setXMLSpecializationType()
end

function SleeperCab.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "openSleepDialog", SleeperCab.openSleepDialog)
    SpecializationUtil.registerFunction(vehicleType, "getSleepCamera", SleeperCab.getSleepCamera)
    SpecializationUtil.registerFunction(vehicleType, "getIsSleeper", SleeperCab.getIsSleeper)
    SpecializationUtil.registerFunction(vehicleType, "onStartSleep", SleeperCab.onStartSleep)
end

function SleeperCab.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad",   SleeperCab)
    SpecializationUtil.registerEventListener(vehicleType, "onDelete", SleeperCab)
end

function SleeperCab:onLoad(savegame)
    self.spec_sleeperCab = self[SleeperCab.SPEC_NAME]

    local spec = self.spec_sleeperCab

    spec.isSleeper = false
    spec.deactivateEngineOnSleep = true
    spec.deactivateLightsOnSleep = true

    if g_sleeperCabManager ~= nil then
        if self.xmlFile:hasProperty("vehicle.enterable.sleeperCab") then
            local validConfiguration, translation, rotation = SleeperCab.getConfigurationData(self.xmlFile, "vehicle.enterable.sleeperCab", self.configurations)

            if validConfiguration then
                spec.isSleeper = true
                spec.sleepCameraNode = self.xmlFile:getValue("vehicle.enterable.sleeperCab#cameraNode", nil, self.components, self.i3dMappings)

                if spec.sleepCameraNode then
                    setUserAttribute(spec.sleepCameraNode, "useCamera", "Boolean", true)

                    if translation ~= nil then
                        setTranslation(spec.sleepCameraNode, unpack(translation))
                    end

                    if rotation ~= nil then
                        setRotation(spec.sleepCameraNode, unpack(rotation))
                    end
                end

                spec.deactivateEngineOnSleep = self.xmlFile:getValue("vehicle.enterable.sleeperCab#deactivateEngineOnSleep", true)
                spec.deactivateLightsOnSleep = self.xmlFile:getValue("vehicle.enterable.sleeperCab#deactivateLightsOnSleep", true)

                g_sleeperCabManager:addVehicle(self)
            end
        elseif g_sleeperCabManager:getIsGlobalVehicle(self.configFileName) then
            local xmlFile = XMLFile.loadIfExists("SleeperCabXML", g_sleeperCabManager.xmlFilename, SleeperCabManager.xmlSchema)

            if xmlFile ~= nil then
                local configFileNameLower = self.configFileName:lower()

                xmlFile:iterate("vehicleSleeperCab.storeItem", function (_, key)
                    local xmlFilename = xmlFile:getValue(key .. "#xmlFilename")

                    if xmlFilename ~= nil and NetworkUtil.convertFromNetworkFilename(xmlFilename):lower() == configFileNameLower then
                        local validConfiguration, translation, rotation = SleeperCab.getConfigurationData(xmlFile, key, self.configurations)

                        if validConfiguration then
                            spec.isSleeper = true

                            if xmlFile:hasProperty(key .. ".camera") then
                                translation = translation or xmlFile:getValue(key .. ".camera#translation", nil, true)
                                rotation = rotation or xmlFile:getValue(key .. ".camera#rotation", nil, true)

                                if translation ~= nil and rotation ~= nil then
                                    local fovy = xmlFile:getValue(key .. ".camera#fovy", 60)
                                    local nearClip = xmlFile:getValue(key .. ".camera#nearClip", 0.1)
                                    local farClip = xmlFile:getValue(key .. ".camera#farClip", 5000)

                                    local sleepCameraNode = createCamera("sleepCamera", fovy, nearClip, farClip)

                                    link(self.components[1].node, sleepCameraNode)

                                    setTranslation(sleepCameraNode, unpack(translation))
                                    setRotation(sleepCameraNode, unpack(rotation))
                                    setUserAttribute(sleepCameraNode, "useCamera", "Boolean", true)

                                    spec.sleepCameraNode = sleepCameraNode
                                end
                            end

                            spec.deactivateEngineOnSleep = xmlFile:getValue(key .. "#deactivateEngineOnSleep", true)
                            spec.deactivateLightsOnSleep = xmlFile:getValue(key .. "#deactivateLightsOnSleep", true)

                            g_sleeperCabManager:addVehicle(self)
                        end

                        return false
                    end
                end)

                xmlFile:delete()
            end
        end
    end
end

function SleeperCab:onDelete()
    if g_sleeperCabManager ~= nil then
        g_sleeperCabManager:removeVehicle(self)
    end
end

function SleeperCab:openSleepDialog(player)
    if self:getIsSleeper() then
        g_sleepManager:showDialog()
    end
end

function SleeperCab:onStartSleep()
    local spec = self.spec_sleeperCab

    if spec ~= nil and spec.isSleeper and not self:getIsAIActive() then
        if spec.deactivateEngineOnSleep and self.getIsMotorStarted ~= nil and self:getIsMotorStarted() then
            self:stopMotor(true)
        end

        if spec.deactivateLightsOnSleep and self.deactivateLights ~= nil then
            self:deactivateLights()
        end
    end
end

function SleeperCab:getSleepCamera()
    local spec = self.spec_sleeperCab

    if spec ~= nil and spec.sleepCameraNode ~= nil then
        return spec.sleepCameraNode
    end

    return nil
end

function SleeperCab:getIsSleeper()
    if self.spec_sleeperCab ~= nil then
        return self.spec_sleeperCab.isSleeper
    end

    return false
end

function SleeperCab.getConfigurationData(xmlFile, key, configurations)
    local configurationName = xmlFile:getValue(key .. ".requiredConfiguration#name")

    if configurationName == nil or configurations == nil then
        return true
    end

    local configuration = configurations[configurationName]

    if configuration ~= nil then
        local validConfiguration, translation, rotation = false, nil, nil

        xmlFile:iterate(key .. ".requiredConfiguration.configuration", function (_, configurationKey)
            local index = xmlFile:getValue(configurationKey .. "#index")

            if index ~= nil and configuration == index then
                validConfiguration = true
                translation = xmlFile:getValue(configurationKey .. "#cameraTranslation", nil, true)
                rotation = xmlFile:getValue(configurationKey .. "#cameraRotation", nil, true)

                return false
            end
        end)

        return validConfiguration, translation, rotation
    end

    return false
end
